@@ -35,14 +35,15 @@ DWORD TraceOption = 0;
 
 HANDLE hDLL;
 
-WSADATA WSAjunk;
 #define AFS_LOGON_EVENT_NAME TEXT("AFS Logon")
 
 void DebugEvent0(char *a) 
 {
     HANDLE h; char *ptbuf[1];
+    
     if (!ISLOGONTRACE(TraceOption))
         return;
+    
     h = RegisterEventSource(NULL, AFS_LOGON_EVENT_NAME);
     ptbuf[0] = a;
     ReportEvent(h, EVENTLOG_INFORMATION_TYPE, 0, 0, NULL, 1, 0, (const char **)ptbuf, NULL);
@@ -77,11 +78,19 @@ BOOLEAN APIENTRY DllEntryPoint(HANDLE dll, DWORD reason, PVOID reserved)
     switch (reason) {
     case DLL_PROCESS_ATTACH:
         /* Initialization Mutex */
-        hInitMutex = CreateMutex(NULL, FALSE, NULL);
+	if (!bInit) {
+	    hInitMutex = CreateMutex(NULL, FALSE, NULL);
+	    SetEnvironmentVariable(DO_NOT_REGISTER_VARNAME, "");
+	}
         break;
 
     case DLL_PROCESS_DETACH:
-        CloseHandle(hInitMutex);
+	/* do nothing on unload because we might 
+	 * be reloaded.
+	 */
+	CloseHandle(hInitMutex);
+	hInitMutex = NULL;
+	bInit = FALSE;
         break;
 
     case DLL_THREAD_ATTACH:
@@ -98,14 +107,28 @@ void AfsLogonInit(void)
 {
     if ( bInit == FALSE ) {
         if ( WaitForSingleObject( hInitMutex, INFINITE ) == WAIT_OBJECT_0 ) {
-            if ( bInit == FALSE ) {
-                rx_Init(0);
-                initAFSDirPath();
-                ka_Init(0);
-                bInit = TRUE;
-            }
-            ReleaseMutex(hInitMutex);
-        }
+	    /* initAFSDirPath() initializes an array and sets a 
+	     * flag so that the initialization can only occur
+	     * once.  No cleanup will be done when the DLL is 
+	     * unloaded so the initialization will not be 
+	     * performed again on a subsequent reload
+	     */
+	    initAFSDirPath();
+
+	    /* ka_Init initializes a number of error tables.
+	     * and then calls ka_CellConfig() which grabs 
+	     * an afsconf_dir structure via afsconf_Open().
+	     * Upon a second attempt to call ka_CellConfig()
+	     * the structure will be released with afsconf_Close()
+	     * and then re-opened.  Could this corrupt memory?
+	     * 
+	     * We only need this if we are not using KFW.
+	     */
+	    if (!KFW_is_available())
+		ka_Init(0);
+	    bInit = TRUE;
+	}
+	ReleaseMutex(hInitMutex);
     }
 }
 
@@ -254,7 +277,7 @@ BOOL IsServiceRunning (void)
 
         CloseServiceHandle (hManager);
     }
-    DebugEvent("AFS AfsLogon - Test Service Running","Return Code[%x] ?Running[%d]",Status.dwCurrentState,(Status.dwCurrentState == SERVICE_RUNNING));
+    DebugEvent("AFS AfsLogon - Test Service Running Return Code[%x] ?Running[%d]",Status.dwCurrentState,(Status.dwCurrentState == SERVICE_RUNNING));
     return (Status.dwCurrentState == SERVICE_RUNNING);
 }   
 
@@ -276,8 +299,8 @@ BOOL IsServiceStartPending (void)
 
         CloseServiceHandle (hManager);
     }
-    DebugEvent("AFS AfsLogon - Test Service Start Pending","Return Code[%x] ?Start Pending[%d]",Status.dwCurrentState,(Status.dwCurrentState == SERVICE_START_PENDING));
-    return (Status.dwCurrentState == SERVICE_RUNNING);
+    DebugEvent("AFS AfsLogon - Test Service Start Pending Return Code[%x] ?Start Pending[%d]",Status.dwCurrentState,(Status.dwCurrentState == SERVICE_START_PENDING));
+    return (Status.dwCurrentState == SERVICE_START_PENDING);
 }   
 
 /* LOOKUPKEYCHAIN: macro to look up the value in the list of keys in order until it's found
@@ -375,7 +398,7 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
             hkDoms = NULL;
         }
     } else
-        DebugEvent("Not opening domain key for [%s]", effDomain);
+        DebugEvent("Not opening domain key");
 
     /* Each individual can either be specified on the domain key, the domains key or in the
        net provider key.  They fail over in that order.  If none is found, we just use the 
@@ -408,7 +431,7 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
     if(ISHIGHSECURITY(opt->LogonOption)) {
         opt->smbName = malloc( MAXRANDOMNAMELEN );
         GenRandomName(opt->smbName);
-    } else {
+    } else if (lpLogonId) {
         /* username and domain for logon session is not necessarily the same as
            username and domain passed into network provider. */
         PSECURITY_LOGON_SESSION_DATA plsd;
@@ -445,6 +468,21 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
 
       bad_strings:
         LsaFreeReturnBuffer(plsd);
+    } else {
+        size_t len;
+
+        DebugEvent("No LUID given. Constructing username using [%s] and [%s]",
+                   username, domain);
+ 
+        len = strlen(username) + strlen(domain) + 2;
+
+        opt->smbName = malloc(len);
+
+        StringCbCopy(opt->smbName, len, username);
+        StringCbCat(opt->smbName, len, "\\");
+        StringCbCat(opt->smbName, len, domain);
+
+        strlwr(opt->smbName);
     }
 
     DebugEvent("Looking up logon script");
@@ -486,7 +524,7 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
         len ++;
 
         wuname = malloc(len * sizeof(WCHAR));
-        MultiByteToWideChar(CP_ACP,0,opt->smbName,-1,wuname,len*sizeof(WCHAR));
+        MultiByteToWideChar(CP_ACP,0,opt->smbName,-1,wuname,(int)(len*sizeof(WCHAR)));
 
         DebugEvent("Username is set for [%S]", wuname);
 
@@ -518,7 +556,7 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
         DebugEvent("After expanding env strings [%S]", regscript);
 
         if(wcsstr(regscript, L"%s")) {
-            dwSize += len * sizeof(WCHAR); /* make room for username expansion */
+            dwSize += (DWORD)(len * sizeof(WCHAR)); /* make room for username expansion */
             regexuscript = (WCHAR *) LocalAlloc(LMEM_FIXED, dwSize);
             hr = StringCbPrintfW(regexuscript, dwSize, regscript, wuname);
         } else {
@@ -576,7 +614,8 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
         DebugEvent("Found TheseCells [%s]", thesecells);
         opt->theseCells = thesecells;
 
-      doneTheseCells:;
+      doneTheseCells:
+        ;
     }
 
   cleanup:
@@ -631,8 +670,8 @@ UnicodeStringToANSI(UNICODE_STRING uInputString, LPSTR lpszOutputString, int nOu
         lpszOutputString[min(uInputString.Length/2,nOutStringLen-1)] = '\0';
         return TRUE;
     }
-    else
-        lpszOutputString[0] = '\0';
+      
+    lpszOutputString[0] = '\0';
     return FALSE;
 }  // UnicodeStringToANSI
 
@@ -651,10 +690,11 @@ DWORD APIENTRY NPLogonNotify(
     char logonDomain[MAX_DOMAIN_LENGTH]="";
     char cell[256]="<non-integrated logon>";
     char homePath[MAX_PATH]="";
+    char szLogonId[128] = "";
 
     MSV1_0_INTERACTIVE_LOGON *IL;
 
-    DWORD code;
+    DWORD code = 0, code2;
 
     int pw_exp;
     char *reason;
@@ -675,15 +715,33 @@ DWORD APIENTRY NPLogonNotify(
     int retryInterval;
     int sleepInterval;
 
+    (void) RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
+                         0, KEY_QUERY_VALUE, &NPKey);
+    LSPsize=sizeof(TraceOption);
+    RegQueryValueEx(NPKey, REG_CLIENT_TRACE_OPTION_PARM, NULL,
+                     &LSPtype, (LPBYTE)&TraceOption, &LSPsize);
+
+    RegCloseKey (NPKey);
+
+    DebugEvent("NPLogonNotify - LoginId(%d,%d)", lpLogonId->HighPart, lpLogonId->LowPart);
+
     /* Make sure the AFS Libraries are initialized */
     AfsLogonInit();
 
     /* Initialize Logon Script to none */
     *lpLogonScript=NULL;
     
-    /* TODO: We should check the value of lpAuthentInfoType before assuming that it is
-       MSV1_0_INTERACTIVE_LOGON though for our purposes KERB_INTERACTIVE_LOGON is
-       co-incidentally equivalent. */
+    /* MSV1_0_INTERACTIVE_LOGON and KERB_INTERACTIVE_LOGON are equivalent for
+     * our purposes */
+
+    if ( wcscmp(lpAuthentInfoType,L"MSV1_0:Interactive") && 
+         wcscmp(lpAuthentInfoType,L"Kerberos:Interactive") )
+    {
+        DebugEvent("Unsupported Authentication Info Type: %S",
+                   lpAuthentInfoType);
+        return 0;
+    }
+
     IL = (MSV1_0_INTERACTIVE_LOGON *) lpAuthentInfo;
 
     /* Are we interactive? */
@@ -692,9 +750,10 @@ DWORD APIENTRY NPLogonNotify(
     /* Convert from Unicode to ANSI */
 
     /*TODO: Use SecureZeroMemory to erase passwords */
-    UnicodeStringToANSI(IL->UserName, uname, MAX_USERNAME_LENGTH);
-    UnicodeStringToANSI(IL->Password, password, MAX_PASSWORD_LENGTH);
-    UnicodeStringToANSI(IL->LogonDomainName, logonDomain, MAX_DOMAIN_LENGTH);
+    if (!UnicodeStringToANSI(IL->UserName, uname, MAX_USERNAME_LENGTH) ||
+	 !UnicodeStringToANSI(IL->Password, password, MAX_PASSWORD_LENGTH) ||
+	 !UnicodeStringToANSI(IL->LogonDomainName, logonDomain, MAX_DOMAIN_LENGTH))
+ 	return 0;
 
     /* Make sure AD-DOMANS sent from login that is sent to us is striped */
     ctemp = strchr(uname, '@');
@@ -708,14 +767,6 @@ DWORD APIENTRY NPLogonNotify(
         }
     }
 
-    (void) RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
-                         0, KEY_QUERY_VALUE, &NPKey);
-    LSPsize=sizeof(TraceOption);
-    RegQueryValueEx(NPKey, REG_CLIENT_TRACE_OPTION_PARM, NULL,
-                     &LSPtype, (LPBYTE)&TraceOption, &LSPsize);
-
-    RegCloseKey (NPKey);
-
     /*
      * Get Logon options
      */
@@ -742,7 +793,7 @@ DWORD APIENTRY NPLogonNotify(
             code = GT_PW_NULL;
             reason = "zero length password is illegal";
             code=0;
-        }       
+        }
 
         /* Get cell name if doing integrated logon.  
            We might overwrite this if we are logging into an AD realm and we find out that
@@ -764,134 +815,148 @@ DWORD APIENTRY NPLogonNotify(
         if (ISREMOTE(opt.flags)) {
             DebugEvent("Is Remote");
             GetAdHomePath(homePath,MAX_PATH,lpLogonId,&opt);
-        }       
+        }
     }
 
     /* loop until AFS is started. */
-    while (TRUE) {
-        DebugEvent("while(TRUE) LogonOption[%x], Service AutoStart[%d]",
-                    opt.LogonOption,afsWillAutoStart);
+    if (afsWillAutoStart) {
+	while (IsServiceRunning() || IsServiceStartPending()) {
+	    DebugEvent("while(autostart) LogonOption[%x], Service AutoStart[%d]",
+			opt.LogonOption,afsWillAutoStart);
 
-        if (ISADREALM(opt.flags)) {
-            code = GetFileCellName(homePath,cell,256);
-            if (!code) {
-                DebugEvent("profile path [%s] is in cell [%s]",homePath,cell);
-            }
-            /* Don't bail out if GetFileCellName failed.
-             * The home dir may not be in AFS after all. 
-             */
-        } else
-            code=0;
+	    if (ISADREALM(opt.flags)) {
+		code = GetFileCellName(homePath,cell,256);
+		if (!code) {
+		    DebugEvent("profile path [%s] is in cell [%s]",homePath,cell);
+		}
+		/* Don't bail out if GetFileCellName failed.
+		 * The home dir may not be in AFS after all. 
+		 */
+	    } else
+		code=0;
 		
-        /* if Integrated Logon  */
-        if (ISLOGONINTEGRATED(opt.LogonOption))
-        {			
-            if ( KFW_is_available() ) {
-                code = KFW_AFS_get_cred(uname, cell, password, 0, opt.smbName, &reason);
-                DebugEvent("KFW_AFS_get_cred  uname=[%s] smbname=[%s] cell=[%s] code=[%d]",uname,opt.smbName,cell,code);
-                if (code == 0 && opt.theseCells) { 
-                    char * principal, *p;
+	    /* if Integrated Logon  */
+	    if (ISLOGONINTEGRATED(opt.LogonOption))
+	    {			
+		if ( KFW_is_available() ) {
+		    code = KFW_AFS_get_cred(uname, cell, password, 0, opt.smbName, &reason);
+		    DebugEvent("KFW_AFS_get_cred  uname=[%s] smbname=[%s] cell=[%s] code=[%d]",
+				uname,opt.smbName,cell,code);
+		    if (code == 0 && opt.theseCells) { 
+			char * principal, *p;
+			size_t len, tlen;
 
-                    principal = (char *)malloc(strlen(uname) + strlen(cell) + 2);
-                    if ( principal ) {
-                        strcpy(principal, uname);
-                        p = principal + strlen(uname);
-                        *p++ = '@';
-                        strcpy(p, cell);
-                        for ( ;*p; p++) {
-                            *p = toupper(*p);
-                        }
+			StringCchLength(cell, MAX_DOMAIN_LENGTH, &tlen);
+			len = tlen;
+			StringCchLength(uname, MAX_USERNAME_LENGTH, &tlen);
+			len += tlen + 2;
 
-                        p = opt.theseCells;
-                        while ( *p ) {
-                            code = KFW_AFS_get_cred(principal, p, 0, 0, opt.smbName, &reason);
-                            DebugEvent("KFW_AFS_get_cred  uname=[%s] smbname=[%s] cell=[%s] code=[%d]",
-                                        principal,opt.smbName,p,code);
-                            p += strlen(p) + 1;
-                        }
-                        
-                        free(principal);
-                    }
-                }
-            } else {
-                code = ka_UserAuthenticateGeneral2(KA_USERAUTH_VERSION+KA_USERAUTH_AUTHENT_LOGON,
-                                                    uname, "", cell, password, opt.smbName, 0, &pw_exp, 0,
-                                                    &reason);
-                DebugEvent("AFS AfsLogon - (INTEGRATED only)ka_UserAuthenticateGeneral2","Code[%x] uname[%s] Cell[%s]",
-                            code,uname,cell);
-            }       
-            if ( code && code != KTC_NOCM && code != KTC_NOCMRPC && !lowercased_name ) {
-                for ( ctemp = uname; *ctemp ; ctemp++) {
-                    *ctemp = tolower(*ctemp);
-                }
-                lowercased_name = TRUE;
-                goto sleeping;
-            }
+			/* tlen is now the length of uname in characters */
+			principal = (char *)malloc(len * sizeof(char));
+			if ( principal ) {
+			    StringCchCopy(principal, len, uname);
+			    p = principal + tlen;
+			    *p++ = '@';
+			    StringCchCopy(p, len - tlen - 1, cell);
+			    for ( ;*p; p++) {
+				*p = toupper(*p);
+			    }
 
-            /* is service started yet?*/
+			    p = opt.theseCells;
+			    while ( *p ) {
+				code2 = KFW_AFS_get_cred(principal, p, 0, 0, opt.smbName, &reason);
+				DebugEvent("KFW_AFS_get_cred  uname=[%s] smbname=[%s] cell=[%s] code=[%d]",
+					    principal,opt.smbName,p,code2);
+				p += strlen(p) + 1;
+			    }
+			    free(principal);
+			}
+		    }
+		} else {
+		    code = ka_UserAuthenticateGeneral2(KA_USERAUTH_VERSION+KA_USERAUTH_AUTHENT_LOGON,
+							uname, "", cell, password, opt.smbName, 0, &pw_exp, 0,
+							&reason);
+		    DebugEvent("AFS AfsLogon - (INTEGRATED only)ka_UserAuthenticateGeneral2 Code[%x] uname[%s] smbname=[%s] Cell[%s] PwExp=[%d] Reason=[%s]",
+				code,uname,opt.smbName,cell,pw_exp,reason?reason:"");
+		}       
+		if ( code && code != KTC_NOCM && code != KTC_NOCMRPC && !lowercased_name ) {
+		    for ( ctemp = uname; *ctemp ; ctemp++) {
+			*ctemp = tolower(*ctemp);
+		    }
+		    lowercased_name = TRUE;
+		    goto sleeping;
+		}
 
-            /* If we've failed because the client isn't running yet and the
-            * client is set to autostart (and therefore it makes sense for
-            * us to wait for it to start) then sleep a while and try again. 
-            * If the error was something else, then give up. */
-            if (code != KTC_NOCM && code != KTC_NOCMRPC || !afsWillAutoStart)
-                break;
-        }
-        else {  
-            /*JUST check to see if its running*/
-            if (IsServiceRunning())
-                break;
-            if (afsWillAutoStart && !IsServiceStartPending()) {
-                code = KTC_NOCMRPC;
-                reason = "AFS Service start failed";
-                break;
-            }
-        }
+		/* is service started yet?*/
 
-        /* If the retry interval has expired and we still aren't
-         * logged in, then just give up if we are not in interactive
-         * mode or the failSilently flag is set, otherwise let the
-         * user know we failed and give them a chance to try again. */
-        if (retryInterval <= 0) {
-            reason = "AFS not running";
-            if (!interactive || opt.failSilently)
-                break;
-            flag = MessageBox(hwndOwner,
-                               "AFS is still starting.  Retry?",
-                               "AFS Logon",
-                               MB_ICONQUESTION | MB_RETRYCANCEL);
-            if (flag == IDCANCEL)
-                break;
+		/* If we've failed because the client isn't running yet and the
+		 * client is set to autostart (and therefore it makes sense for
+		 * us to wait for it to start) then sleep a while and try again. 
+		 * If the error was something else, then give up. */
+		if (code != KTC_NOCM && code != KTC_NOCMRPC)
+		    break;
+	    }
+	    else {  
+		/*JUST check to see if its running*/
+		if (IsServiceRunning())
+		    break;
+		if (!IsServiceStartPending()) {
+		    code = KTC_NOCMRPC;
+		    reason = "AFS Service start failed";
+		    break;
+		}
+	    }
 
-            /* Wait just a little while and try again */
-            retryInterval = opt.retryInterval;
-        }
+	    /* If the retry interval has expired and we still aren't
+	     * logged in, then just give up if we are not in interactive
+	     * mode or the failSilently flag is set, otherwise let the
+	     * user know we failed and give them a chance to try again. */
+	    if (retryInterval <= 0) {
+		reason = "AFS not running";
+		if (!interactive || opt.failSilently)
+		    break;
+		flag = MessageBox(hwndOwner,
+				   "AFS is still starting.  Retry?",
+				   "AFS Logon",
+				   MB_ICONQUESTION | MB_RETRYCANCEL);
+		if (flag == IDCANCEL)
+		    break;
 
-      sleeping:
-        Sleep(sleepInterval * 1000);
-        retryInterval -= sleepInterval;
+		/* Wait just a little while and try again */
+		retryInterval = opt.retryInterval;
+	    }
+
+	  sleeping:
+	    Sleep(sleepInterval * 1000);
+	    retryInterval -= sleepInterval;
+	}
+    }
+    DebugEvent("while loop exited");
+    /* remove any kerberos 5 tickets currently held by the SYSTEM account
+     * for this user 
+     */
+    if (ISLOGONINTEGRATED(opt.LogonOption) && KFW_is_available()) {
+	sprintf(szLogonId,"%d.%d",lpLogonId->HighPart, lpLogonId->LowPart);
+	KFW_AFS_copy_cache_to_system_file(uname, szLogonId);
+
+	KFW_AFS_destroy_tickets_for_principal(uname);
     }
 
-    DebugEvent("while loop exited");
-    /* remove any kerberos 5 tickets currently held by the SYSTEM account */
-    if ( KFW_is_available() )
-        KFW_AFS_destroy_tickets_for_cell(cell);
-
     if (code) {
-        char msg[128];
-        HANDLE h;
-        char *ptbuf[1];
+	char msg[128];
+	HANDLE h;
+	char *ptbuf[1];
 
-        StringCbPrintf(msg, sizeof(msg), "Integrated login failed: %s", reason);
+	StringCbPrintf(msg, sizeof(msg), "Integrated login failed: %s", reason);
 
-        if (ISLOGONINTEGRATED(opt.LogonOption) && interactive && !opt.failSilently)
-            MessageBox(hwndOwner, msg, "AFS Logon", MB_OK);
+	if (ISLOGONINTEGRATED(opt.LogonOption) && interactive && !opt.failSilently)
+	    MessageBox(hwndOwner, msg, "AFS Logon", MB_OK);
 
-        h = RegisterEventSource(NULL, AFS_LOGON_EVENT_NAME);
-        ptbuf[0] = msg;
-        ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1008, NULL,
-                     1, 0, ptbuf, NULL);
-        DeregisterEventSource(h);
+	h = RegisterEventSource(NULL, AFS_LOGON_EVENT_NAME);
+	ptbuf[0] = msg;
+	ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1008, NULL,
+		     1, 0, ptbuf, NULL);
+	DeregisterEventSource(h);
 	    
         code = MapAuthError(code);
         SetLastError(code);
@@ -988,79 +1053,155 @@ VOID AFS_Logoff_Event( PWLX_NOTIFICATION_INFO pInfo )
     DWORD  len = 1024;
     PTOKEN_USER  tokenUser = NULL;
     DWORD  retLen;
+    DWORD LSPtype, LSPsize;
+    HKEY NPKey;
+    DWORD LogoffPreserveTokens = 0;
+    LogonOptions_t opt;
 
     /* Make sure the AFS Libraries are initialized */
     AfsLogonInit();
 
     DebugEvent0("AFS_Logoff_Event - Start");
 
-    if (!GetTokenInformation(pInfo->hToken, TokenUser, NULL, 0, &retLen))
-    {
-        if ( GetLastError() == ERROR_INSUFFICIENT_BUFFER ) {
-            tokenUser = (PTOKEN_USER) LocalAlloc(LPTR, retLen);
+    (void) RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
+                         0, KEY_QUERY_VALUE, &NPKey);
+    LSPsize=sizeof(LogoffPreserveTokens);
+    RegQueryValueEx(NPKey, REG_CLIENT_LOGOFF_TOKENS_PARM, NULL,
+                     &LSPtype, (LPBYTE)&LogoffPreserveTokens, &LSPsize);
+    RegCloseKey (NPKey);
 
-            if (!GetTokenInformation(pInfo->hToken, TokenUser, tokenUser, retLen, &retLen))
-            {
-                DebugEvent("AFS_Logoff_Event - GetTokenInformation failed: GLE = %lX", GetLastError());
-            }
-        }
-    }
+    if (!LogoffPreserveTokens) {
+	memset(&opt, 0, sizeof(LogonOptions_t));
 
-    /* We can't use pInfo->Domain for the domain since in the cross realm case 
-     * this is source domain and not the destination domain.
-     */
-    if (QueryAdHomePathFromSid( profileDir, sizeof(profileDir), tokenUser->User.Sid, pInfo->Domain)) {
-        WCHAR Domain[64]=L"";
-        GetLocalShortDomain(Domain, sizeof(Domain));
-        if (QueryAdHomePathFromSid( profileDir, sizeof(profileDir), tokenUser->User.Sid, Domain)) {
-            if (NetUserGetProfilePath(pInfo->Domain, pInfo->UserName, profileDir, len))
-                GetUserProfileDirectory(pInfo->hToken, profileDir, &len);
-        }
-    }
-    
-    if (strlen(profileDir)) {
-        DebugEvent("AFS_Logoff_Event - Profile Directory: %s", profileDir);
-        if (!IsPathInAfs(profileDir)) {
-            if (code = ktc_ForgetAllTokens())
-                DebugEvent("AFS_Logoff_Event - ForgetAllTokens failed [%lX]",code);
-            else
-                DebugEvent0("AFS_Logoff_Event - ForgetAllTokens succeeded");
-        } else {
-            DebugEvent0("AFS_Logoff_Event - Tokens left in place; profile in AFS");
-        }
+	if (pInfo->UserName && pInfo->Domain) {
+	    char username[MAX_USERNAME_LENGTH] = "";
+	    char domain[MAX_DOMAIN_LENGTH] = "";
+	    size_t szlen = 0;
+
+	    StringCchLengthW(pInfo->UserName, MAX_USERNAME_LENGTH, &szlen);
+	    WideCharToMultiByte(CP_UTF8, 0, pInfo->UserName, szlen,
+				 username, sizeof(username), NULL, NULL);
+
+	    StringCchLengthW(pInfo->Domain, MAX_DOMAIN_LENGTH, &szlen);
+	    WideCharToMultiByte(CP_UTF8, 0, pInfo->Domain, szlen,
+				 domain, sizeof(domain), NULL, NULL);
+
+	    GetDomainLogonOptions(NULL, username, domain, &opt);
+	}
+
+        if (ISREMOTE(opt.flags)) {
+	    if (!GetTokenInformation(pInfo->hToken, TokenUser, NULL, 0, &retLen))
+	    {
+		if ( GetLastError() == ERROR_INSUFFICIENT_BUFFER ) {
+		    tokenUser = (PTOKEN_USER) LocalAlloc(LPTR, retLen);
+
+		    if (!GetTokenInformation(pInfo->hToken, TokenUser, tokenUser, retLen, &retLen))
+		    {
+			DebugEvent("AFS_Logoff_Event - GetTokenInformation failed: GLE = %lX", GetLastError());
+		    }
+		}
+	    }
+
+	    /* We can't use pInfo->Domain for the domain since in the cross realm case 
+	     * this is source domain and not the destination domain.
+	     */
+	    if (QueryAdHomePathFromSid( profileDir, sizeof(profileDir), tokenUser->User.Sid, pInfo->Domain)) {
+		WCHAR Domain[64]=L"";
+		GetLocalShortDomain(Domain, sizeof(Domain));
+		if (QueryAdHomePathFromSid( profileDir, sizeof(profileDir), tokenUser->User.Sid, Domain)) {
+		    if (NetUserGetProfilePath(pInfo->Domain, pInfo->UserName, profileDir, len))
+			GetUserProfileDirectory(pInfo->hToken, profileDir, &len);
+		}
+	    }
+
+	    if (strlen(profileDir)) {
+		DebugEvent("AFS_Logoff_Event - Profile Directory: %s", profileDir);
+		if (!IsPathInAfs(profileDir)) {
+		    if (code = ktc_ForgetAllTokens())
+			DebugEvent("AFS_Logoff_Event - ForgetAllTokens failed [%lX]",code);
+		    else
+			DebugEvent0("AFS_Logoff_Event - ForgetAllTokens succeeded");
+		} else {
+		    DebugEvent0("AFS_Logoff_Event - Tokens left in place; profile in AFS");
+		}
+	    } else {
+		DebugEvent0("AFS_Logoff_Event - Unable to load profile");
+	    }
+
+	    if ( tokenUser )
+		LocalFree(tokenUser);
+	} else {
+	    DebugEvent0("AFS_Logoff_Event - Local Logon");
+	    if (code = ktc_ForgetAllTokens())
+		DebugEvent("AFS_Logoff_Event - ForgetAllTokens failed [%lX]",code);
+	    else
+		DebugEvent0("AFS_Logoff_Event - ForgetAllTokens succeeded");
+	}
     } else {
-        DebugEvent0("AFS_Logoff_Event - Unable to load profile");
+	DebugEvent0("AFS_Logoff_Event - Preserving Tokens");
     }
 
-    if ( tokenUser )
-        LocalFree(tokenUser);
-
     DebugEvent0("AFS_Logoff_Event - End");
 }   
 
 VOID AFS_Logon_Event( PWLX_NOTIFICATION_INFO pInfo )
 {
-    DWORD code;
     TCHAR profileDir[1024] = TEXT("");
     DWORD  len = 1024;
     PTOKEN_USER  tokenUser = NULL;
     DWORD  retLen;
-    HANDLE hToken;
-
     WCHAR szUserW[128] = L"";
     char  szUserA[128] = "";
     char  szClient[MAX_PATH];
     char szPath[MAX_PATH] = "";
     NETRESOURCE nr;
     DWORD res;
-    DWORD gle;
     DWORD dwSize;
+    LogonOptions_t opt;
 
     /* Make sure the AFS Libraries are initialized */
     AfsLogonInit();
 
     DebugEvent0("AFS_Logon_Event - Start");
 
+    DebugEvent("AFS_Logon_Event Process ID: %d",GetCurrentProcessId());
+
+    memset(&opt, 0, sizeof(LogonOptions_t));
+
+    if (pInfo->UserName && pInfo->Domain) {
+        char username[MAX_USERNAME_LENGTH] = "";
+        char domain[MAX_DOMAIN_LENGTH] = "";
+        size_t szlen = 0;
+
+	DebugEvent0("AFS_Logon_Event - pInfo UserName and Domain");
+
+        StringCchLengthW(pInfo->UserName, MAX_USERNAME_LENGTH, &szlen);
+        WideCharToMultiByte(CP_UTF8, 0, pInfo->UserName, szlen,
+                            username, sizeof(username), NULL, NULL);
+        
+        StringCchLengthW(pInfo->Domain, MAX_DOMAIN_LENGTH, &szlen);
+        WideCharToMultiByte(CP_UTF8, 0, pInfo->Domain, szlen,
+                            domain, sizeof(domain), NULL, NULL);
+
+	DebugEvent0("AFS_Logon_Event - Calling GetDomainLogonOptions");
+        GetDomainLogonOptions(NULL, username, domain, &opt);
+    } else {
+	if (!pInfo->UserName)
+	    DebugEvent0("AFS_Logon_Event - No pInfo->UserName");
+	if (!pInfo->Domain)
+	    DebugEvent0("AFS_Logon_Event - No pInfo->Domain");
+    }
+
+    DebugEvent("AFS_Logon_Event - opt.LogonOption = %lX opt.flags = %lX", 
+		opt.LogonOption, opt.flags);
+
+    if (!ISLOGONINTEGRATED(opt.LogonOption) || !ISREMOTE(opt.flags)) {
+        DebugEvent0("AFS_Logon_Event - Logon is not integrated or not remote");
+        goto done_logon_event;
+    }
+
+    DebugEvent0("AFS_Logon_Event - Calling GetTokenInformation");
+
     if (!GetTokenInformation(pInfo->hToken, TokenUser, NULL, 0, &retLen))
     {
         if ( GetLastError() == ERROR_INSUFFICIENT_BUFFER ) {
@@ -1091,6 +1232,7 @@ VOID AFS_Logon_Event( PWLX_NOTIFICATION_INFO pInfo )
         DebugEvent0("AFS_Logon_Event - Unable to load profile");
     }
 
+  done_logon_event:
     dwSize = sizeof(szUserA);
     if (!KFW_AFS_get_lsa_principal(szUserA, &dwSize)) {
         StringCbPrintfW(szUserW, sizeof(szUserW), L"%s\\%s", pInfo->Domain, pInfo->UserName);
@@ -1123,3 +1265,144 @@ VOID AFS_Logon_Event( PWLX_NOTIFICATION_INFO pInfo )
     DebugEvent0("AFS_Logon_Event - End");
 }
 
+static BOOL
+GetSecurityLogonSessionData(HANDLE hToken, PSECURITY_LOGON_SESSION_DATA * ppSessionData)
+{
+    NTSTATUS Status = 0;
+    TOKEN_STATISTICS Stats;
+    DWORD   ReqLen;
+    BOOL    Success;
+
+    if (!ppSessionData)
+        return FALSE;
+    *ppSessionData = NULL;
+
+    Success = GetTokenInformation( hToken, TokenStatistics, &Stats, sizeof(TOKEN_STATISTICS), &ReqLen );
+    if ( !Success )
+        return FALSE;
+
+    Status = LsaGetLogonSessionData( &Stats.AuthenticationId, ppSessionData );
+    if ( FAILED(Status) || !ppSessionData )
+        return FALSE;
+
+    return TRUE;
+}
+
+VOID KFW_Logon_Event( PWLX_NOTIFICATION_INFO pInfo )
+{
+    WCHAR szUserW[128] = L"";
+    char  szUserA[128] = "";
+    char szPath[MAX_PATH] = "";
+    char szLogonId[128] = "";
+    DWORD count;
+    char filename[MAX_PATH] = "";
+    char newfilename[MAX_PATH] = "";
+    char commandline[MAX_PATH+256] = "";
+    STARTUPINFO startupinfo;
+    PROCESS_INFORMATION procinfo;
+    HANDLE hf = INVALID_HANDLE_VALUE;
+
+    LUID LogonId = {0, 0};
+    PSECURITY_LOGON_SESSION_DATA pLogonSessionData = NULL;
+
+    HKEY hKey1 = NULL, hKey2 = NULL;
+
+    /* Make sure the KFW Libraries are initialized */
+    AfsLogonInit();
+
+    DebugEvent0("KFW_Logon_Event - Start");
+
+    GetSecurityLogonSessionData( pInfo->hToken, &pLogonSessionData );
+
+    if ( pLogonSessionData ) {
+        LogonId = pLogonSessionData->LogonId;
+        DebugEvent("KFW_Logon_Event - LogonId(%d,%d)", LogonId.HighPart, LogonId.LowPart);
+
+        sprintf(szLogonId,"%d.%d",LogonId.HighPart, LogonId.LowPart);
+        LsaFreeReturnBuffer( pLogonSessionData );
+    } else {
+        DebugEvent0("KFW_Logon_Event - Unable to determine LogonId");
+        return;
+    }
+
+    count = GetEnvironmentVariable("TEMP", filename, sizeof(filename));
+    if ( count > sizeof(filename) || count == 0 ) {
+        GetWindowsDirectory(filename, sizeof(filename));
+    }
+
+    count = GetEnvironmentVariable("TEMP", filename, sizeof(filename));
+    if ( count > sizeof(filename) || count == 0 ) {
+        GetWindowsDirectory(filename, sizeof(filename));
+    }
+
+    if ( strlen(filename) + strlen(szLogonId) + 2 > sizeof(filename) ) {
+        DebugEvent0("KFW_Logon_Event - filename too long");
+	return;
+    }
+
+    strcat(filename, "\\");
+    strcat(filename, szLogonId);    
+
+    hf = CreateFile(filename, FILE_ALL_ACCESS, 0, NULL, OPEN_EXISTING, 
+		     FILE_ATTRIBUTE_NORMAL, NULL);
+    if (hf == INVALID_HANDLE_VALUE) {
+	DebugEvent0("KFW_Logon_Event - file cannot be opened");
+ 	return;
+    }
+    CloseHandle(hf);
+ 
+    if (KFW_AFS_set_file_cache_dacl(filename, pInfo->hToken)) {
+	DebugEvent0("KFW_Logon_Event - unable to set dacl");
+ 	DeleteFile(filename);
+ 	return;
+    }
+ 
+    if (KFW_AFS_obtain_user_temp_directory(pInfo->hToken, newfilename, sizeof(newfilename))) {
+	DebugEvent0("KFW_Logon_Event - unable to obtain temp directory");
+ 	return;
+    }
+
+    if ( strlen(newfilename) + strlen(szLogonId) + 2 > sizeof(newfilename) ) {
+        DebugEvent0("KFW_Logon_Event - new filename too long");
+	return;
+    }
+
+    strcat(newfilename, "\\");
+    strcat(newfilename, szLogonId);    
+
+    if (!MoveFileEx(filename, newfilename, 
+		     MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING | MOVEFILE_WRITE_THROUGH)) {
+        DebugEvent("KFW_Logon_Event - MoveFileEx failed GLE = 0x%x", GetLastError());
+	return;
+    }
+
+    sprintf(commandline, "afscpcc.exe \"%s\"", newfilename);
+
+    GetStartupInfo(&startupinfo);
+    if (CreateProcessAsUser( pInfo->hToken,
+                             "afscpcc.exe",
+                             commandline,
+                             NULL,
+                             NULL,
+                             FALSE,
+                             CREATE_NEW_PROCESS_GROUP | DETACHED_PROCESS,
+                             NULL,
+                             NULL,
+                             &startupinfo,
+                             &procinfo)) 
+    {
+	DebugEvent("KFW_Logon_Event - CommandLine %s", commandline);
+
+	WaitForSingleObject(procinfo.hProcess, 30000);
+
+	CloseHandle(procinfo.hThread);
+	CloseHandle(procinfo.hProcess);
+    } else {
+	DebugEvent0("KFW_Logon_Event - CreateProcessFailed");
+    }
+
+    DeleteFile(filename);
+
+    DebugEvent0("KFW_Logon_Event - End");
+}
+
