@@ -21,14 +21,28 @@
 #include <malloc.h>
 #include <string.h>
 
+#include "afsd.h"
 #include <osi.h>
 #include <rx/rx.h>
-#include "afsd.h"
 
 osi_rwlock_t cm_serverLock;
 
 cm_server_t *cm_allServersp;
 
+void
+cm_ForceNewConnectionsAllServers(void)
+{
+    cm_server_t *tsp;
+
+    lock_ObtainRead(&cm_serverLock);
+    for (tsp = cm_allServersp; tsp; tsp = tsp->allNextp) {
+        cm_GetServerNoLock(tsp);
+	cm_ForceNewConnections(tsp);
+        cm_PutServerNoLock(tsp);
+    }
+    lock_ReleaseRead(&cm_serverLock);
+}
+
 void 
 cm_PingServer(cm_server_t *tsp)
 {
@@ -212,6 +226,26 @@ void cm_PutServerNoLock(cm_server_t *serverp)
     osi_assert(serverp->refCount-- > 0);
 }
 
+void cm_SetServerNo64Bit(cm_server_t * serverp, int no64bit)
+{
+    lock_ObtainMutex(&serverp->mx);
+    if (no64bit)
+        serverp->flags |= CM_SERVERFLAG_NO64BIT;
+    else
+        serverp->flags &= ~CM_SERVERFLAG_NO64BIT;
+    lock_ReleaseMutex(&serverp->mx);
+}
+
+void cm_SetServerNoInlineBulk(cm_server_t * serverp, int no)
+{
+    lock_ObtainMutex(&serverp->mx);
+    if (no)
+        serverp->flags |= CM_SERVERFLAG_NOINLINEBULK;
+    else
+        serverp->flags &= ~CM_SERVERFLAG_NOINLINEBULK;
+    lock_ReleaseMutex(&serverp->mx);
+}
+
 void cm_SetServerPrefs(cm_server_t * serverp)
 {
     unsigned long	serverAddr; 	/* in host byte order */
