@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/DARWIN/osi_file.c,v 1.8 2003/07/15 23:14:17 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/DARWIN/osi_file.c,v 1.12.2.1 2006/08/20 22:15:16 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -32,6 +32,9 @@ afs_InitDualFSCacheOps(struct vnode *vp)
 {
     int code;
     static int inited = 0;
+#ifdef AFS_DARWIN80_ENV
+    char *buffer = (char*)_MALLOC(MFSNAMELEN, M_TEMP, M_WAITOK);
+#endif
 
     if (inited)
 	return;
@@ -39,24 +42,47 @@ afs_InitDualFSCacheOps(struct vnode *vp)
 
     if (vp == NULL)
 	return;
+#ifdef AFS_DARWIN80_ENV
+    vfs_name(vnode_mount(vp), buffer);
+    if (strncmp("hfs", buffer, 3) == 0)
+#else
     if (strncmp("hfs", vp->v_mount->mnt_vfc->vfc_name, 3) == 0)
+#endif
 	afs_CacheFSType = AFS_APPL_HFS_CACHE;
+#ifdef AFS_DARWIN80_ENV
+    else if (strncmp("ufs", buffer, 3) == 0)
+#else
     else if (strncmp("ufs", vp->v_mount->mnt_vfc->vfc_name, 3) == 0)
+#endif
 	afs_CacheFSType = AFS_APPL_UFS_CACHE;
     else
 	osi_Panic("Unknown cache vnode type\n");
+#ifdef AFS_DARWIN80_ENV
+    _FREE(buffer, M_TEMP);
+#endif
 }
 
 ino_t
-VnodeToIno(vnode_t * avp)
+VnodeToIno(vnode_t avp)
 {
     unsigned long ret;
 
+#ifndef AFS_DARWIN80_ENV
     if (afs_CacheFSType == AFS_APPL_UFS_CACHE) {
 	struct inode *ip = VTOI(avp);
 	ret = ip->i_number;
     } else if (afs_CacheFSType == AFS_APPL_HFS_CACHE) {
-#ifndef VTOH
+#endif
+#if defined(AFS_DARWIN80_ENV) 
+	struct vattr va;
+	VATTR_INIT(&va);
+        VATTR_WANTED(&va, va_fileid);
+	if (vnode_getattr(avp, &va, afs_osi_ctxtp))
+	    osi_Panic("VOP_GETATTR failed in VnodeToIno\n");
+        if (!VATTR_ALL_SUPPORTED(&va))
+	    osi_Panic("VOP_GETATTR unsupported fileid in VnodeToIno\n");
+	ret = va.va_fileid;
+#elif !defined(VTOH)
 	struct vattr va;
 	if (VOP_GETATTR(avp, &va, &afs_osi_cred, current_proc()))
 	    osi_Panic("VOP_GETATTR failed in VnodeToIno\n");
@@ -65,22 +91,33 @@ VnodeToIno(vnode_t * avp)
 	struct hfsnode *hp = VTOH(avp);
 	ret = H_FILEID(hp);
 #endif
+#ifndef AFS_DARWIN80_ENV
     } else
 	osi_Panic("VnodeToIno called before cacheops initialized\n");
+#endif
     return ret;
 }
 
 
 dev_t
-VnodeToDev(vnode_t * avp)
+VnodeToDev(vnode_t avp)
 {
-
-
+#ifndef AFS_DARWIN80_ENV
     if (afs_CacheFSType == AFS_APPL_UFS_CACHE) {
 	struct inode *ip = VTOI(avp);
 	return ip->i_dev;
     } else if (afs_CacheFSType == AFS_APPL_HFS_CACHE) {
-#ifndef VTOH			/* slow, but works */
+#endif
+#if defined(AFS_DARWIN80_ENV) 
+	struct vattr va;
+        VATTR_INIT(&va);
+        VATTR_WANTED(&va, va_fsid);
+	if (vnode_getattr(avp, &va, afs_osi_ctxtp))
+	    osi_Panic("VOP_GETATTR failed in VnodeToDev\n");
+        if (!VATTR_ALL_SUPPORTED(&va))
+	    osi_Panic("VOP_GETATTR unsupported fsid in VnodeToIno\n");
+	return va.va_fsid;	/* XXX they say it's the dev.... */
+#elif !defined(VTOH)
 	struct vattr va;
 	if (VOP_GETATTR(avp, &va, &afs_osi_cred, current_proc()))
 	    osi_Panic("VOP_GETATTR failed in VnodeToDev\n");
@@ -89,8 +126,10 @@ VnodeToDev(vnode_t * avp)
 	struct hfsnode *hp = VTOH(avp);
 	return H_DEV(hp);
 #endif
+#ifndef AFS_DARWIN80_ENV
     } else
 	osi_Panic("VnodeToDev called before cacheops initialized\n");
+#endif
 }
 
 void *
@@ -115,14 +154,18 @@ osi_UFSOpen(afs_int32 ainode)
     }
     afile = (struct osi_file *)osi_AllocSmallSpace(sizeof(struct osi_file));
     AFS_GUNLOCK();
+#ifndef AFS_DARWIN80_ENV
     if (afs_CacheFSType == AFS_APPL_HFS_CACHE)
 	code = igetinode(afs_cacheVfsp, (dev_t) cacheDev.dev, &ainode, &vp, &va, &dummy);	/* XXX hfs is broken */
     else if (afs_CacheFSType == AFS_APPL_UFS_CACHE)
+#endif
 	code =
 	    igetinode(afs_cacheVfsp, (dev_t) cacheDev.dev, (ino_t) ainode,
 		      &vp, &va, &dummy);
+#ifndef AFS_DARWIN80_ENV
     else
 	panic("osi_UFSOpen called before cacheops initialized\n");
+#endif
     AFS_GLOCK();
     if (code) {
 	osi_FreeSmallSpace(afile);
@@ -144,7 +187,18 @@ afs_osi_Stat(register struct osi_file *afile, register struct osi_stat *astat)
     AFS_STATCNT(osi_Stat);
     MObtainWriteLock(&afs_xosi, 320);
     AFS_GUNLOCK();
+#ifdef AFS_DARWIN80_ENV
+    VATTR_INIT(&tvattr);
+    VATTR_WANTED(&tvattr, va_size);
+    VATTR_WANTED(&tvattr, va_blocksize);
+    VATTR_WANTED(&tvattr, va_mtime);
+    VATTR_WANTED(&tvattr, va_atime);
+    code = vnode_getattr(afile->vnode, &tvattr, afs_osi_ctxtp);
+    if (code == 0 && !VATTR_ALL_SUPPORTED(&tvattr))
+       code = EINVAL;
+#else
     code = VOP_GETATTR(afile->vnode, &tvattr, &afs_osi_cred, current_proc());
+#endif
     AFS_GLOCK();
     if (code == 0) {
 	astat->size = tvattr.va_size;
@@ -161,7 +215,11 @@ osi_UFSClose(register struct osi_file *afile)
 {
     AFS_STATCNT(osi_Close);
     if (afile->vnode) {
+#ifdef AFS_DARWIN80_ENV
+        vnode_close(afile->vnode, O_RDWR, afs_osi_ctxtp);
+#else
 	AFS_RELE(afile->vnode);
+#endif
     }
 
     osi_FreeSmallSpace(afile);
@@ -185,20 +243,35 @@ osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
     if (code || tstat.size <= asize)
 	return code;
     MObtainWriteLock(&afs_xosi, 321);
+    AFS_GUNLOCK();
+#ifdef AFS_DARWIN80_ENV
+    VATTR_INIT(&tvattr);
+    VATTR_SET(&tvattr, va_size, asize);
+    code = vnode_setattr(afile->vnode, &tvattr, afs_osi_ctxtp);
+#else
     VATTR_NULL(&tvattr);
     tvattr.va_size = asize;
-    AFS_GUNLOCK();
     code = VOP_SETATTR(afile->vnode, &tvattr, &afs_osi_cred, current_proc());
+#endif
     AFS_GLOCK();
     MReleaseWriteLock(&afs_xosi);
     return code;
 }
 
 void
+#ifdef AFS_DARWIN80_ENV
+osi_DisableAtimes(vnode_t avp)
+#else
 osi_DisableAtimes(struct vnode *avp)
+#endif
 {
+#ifdef AFS_DARWIN80_ENV
+    struct vnode_attr vap;
 
-
+    VATTR_INIT(&vap);
+    VATTR_CLEAR_SUPPORTED(&vap, va_access_time);
+    vnode_setattr(avp, &vap, afs_osi_ctxtp);
+#else
     if (afs_CacheFSType == AFS_APPL_UFS_CACHE) {
 	struct inode *ip = VTOI(avp);
 	ip->i_flag &= ~IN_ACCESS;
@@ -209,6 +282,7 @@ osi_DisableAtimes(struct vnode *avp)
 	hp->h_nodeflags &= ~IN_ACCESS;
     }
 #endif
+#endif
 }
 
 
@@ -218,8 +292,11 @@ afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
 	     afs_int32 asize)
 {
     struct AFS_UCRED *oldCred;
-    unsigned int resid;
+    afs_size_t resid;
     register afs_int32 code;
+#ifdef AFS_DARWIN80_ENV
+    uio_t uio;
+#endif
     AFS_STATCNT(osi_Read);
 
     /**
@@ -236,9 +313,17 @@ afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
     if (offset != -1)
 	afile->offset = offset;
     AFS_GUNLOCK();
+#ifdef AFS_DARWIN80_ENV
+    uio=uio_create(1, afile->offset, AFS_UIOSYS, UIO_READ);
+    uio_addiov(uio, CAST_USER_ADDR_T(aptr), asize);
+    code = VNOP_READ(afile->vnode, uio, IO_UNIT, afs_osi_ctxtp);
+    resid = AFS_UIO_RESID(uio);
+    uio_free(uio);
+#else
     code =
 	gop_rdwr(UIO_READ, afile->vnode, (caddr_t) aptr, asize, afile->offset,
 		 AFS_UIOSYS, IO_UNIT, &afs_osi_cred, &resid);
+#endif
     AFS_GLOCK();
     if (code == 0) {
 	code = asize - resid;
@@ -258,8 +343,11 @@ afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
 	      afs_int32 asize)
 {
     struct AFS_UCRED *oldCred;
-    unsigned int resid;
+    afs_size_t resid;
     register afs_int32 code;
+#ifdef AFS_DARWIN80_ENV
+    uio_t uio;
+#endif
     AFS_STATCNT(osi_Write);
     if (!afile)
 	osi_Panic("afs_osi_Write called with null param");
@@ -267,10 +355,18 @@ afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
 	afile->offset = offset;
     {
 	AFS_GUNLOCK();
+#ifdef AFS_DARWIN80_ENV
+        uio=uio_create(1, afile->offset, AFS_UIOSYS, UIO_WRITE);
+        uio_addiov(uio, CAST_USER_ADDR_T(aptr), asize);
+        code = VNOP_WRITE(afile->vnode, uio, IO_UNIT, afs_osi_ctxtp);
+        resid = AFS_UIO_RESID(uio);
+        uio_free(uio);
+#else
 	code =
 	    gop_rdwr(UIO_WRITE, afile->vnode, (caddr_t) aptr, asize,
 		     afile->offset, AFS_UIOSYS, IO_UNIT, &afs_osi_cred,
 		     &resid);
+#endif
 	AFS_GLOCK();
     }
     if (code == 0) {
