@@ -86,13 +86,9 @@ extern int afs_shuttingdown;
 #define	AFS_NRXPACKETS	80
 #define	AFS_RXDEADTIME	50
 #define AFS_HARDDEADTIME	120
-#define	AFS_IDLEDEADTIME	50
-#define AFS_BLKBITS	12
-#define AFS_BLKSIZE	(1 << AFS_BLKBITS)
 
 extern afs_int32 afs_rx_deadtime;
 extern afs_int32 afs_rx_harddead;
-extern afs_int32 afs_rx_idledead;
 
 struct sysname_info {
     char *name;
@@ -183,14 +179,11 @@ struct vrequest {
     afs_int32 uid;		/* user id making the request */
     afs_int32 busyCount;	/* how many busies we've seen so far */
     afs_int32 flags;		/* things like O_SYNC, O_NONBLOCK go here */
-    char initd;			/* if non-zero, Error fields meaningful */
+    char initd;			/* if non-zero, non-uid fields meaningful */
     char accessError;		/* flags for overriding error return code */
     char volumeError;		/* encountered a missing or busy volume */
     char networkError;		/* encountered network problems */
     char permWriteError;	/* fileserver returns permenent error. */
-    char tokenError;            /* a token error other than expired. */
-    char idleError;             /* the server idled too long */
-    char skipserver[MAXHOSTS];
 };
 #define VOLMISSING 1
 #define VOLBUSY 2
@@ -231,6 +224,7 @@ struct cell {
     time_t timeout;		/* data expire time, if non-zero */
     struct cell_name *cnamep;	/* pointer to our cell_name */
     afs_rwlock_t lock;		/* protects cell data */
+    unsigned char cellHandle[16];	/* deterministic handle for this cell */
 };
 
 struct cell_name {
@@ -255,6 +249,7 @@ struct cell_alias {
 #define UPrimary        4	/* on iff primary identity */
 #define UNeedsReset	8	/* needs afs_ResetAccessCache call done */
 #define UPAGCounted    16	/* entry seen during PAG search (for stats) */
+#define UNFSGetCreds   32	/* getting creds for NFS client */
 /* A flag used by afs_GCPAGs to keep track of
  * which entries in afs_users need to be deleted.
  * The lifetime of its presence in the table is the
@@ -273,9 +268,6 @@ enum { AFS_GCPAGS_NOTCOMPILED = 0, AFS_GCPAGS_OK =
 extern afs_int32 afs_gcpags;
 extern afs_int32 afs_gcpags_procsize;
 extern afs_int32 afs_bkvolpref;
-extern char afs_cachebasedir[1024];
-extern afs_int32 afs_numcachefiles;
-extern afs_int32 afs_numfilesperdir;
 
 struct unixuser {
     struct unixuser *next;	/* next hash pointer */
@@ -289,6 +281,7 @@ struct unixuser {
     char *stp;			/* pointer to ticket itself */
     struct ClearToken ct;
     struct afs_exporter *exporter;	/* more info about the exporter for the remote user */
+    void *cellinfo;             /* pointer to cell info (PAG manager only) */
 };
 
 struct conn {
@@ -549,16 +542,13 @@ struct SimpleLocks {
 #endif
 #define CUnique		0x00001000	/* vc's uniquifier - latest unifiquier for fid */
 #define CForeign	0x00002000	/* this is a non-afs vcache */
+#define CReadDir	0x00004000	/* readdir in progress */
 #define CUnlinked	0x00010000
 #define CBulkStat	0x00020000	/* loaded by a bulk stat, and not ref'd since */
 #define CUnlinkedDel	0x00040000
 #define CVFlushed	0x00080000
 #define CCore1		0x00100000	/* osf1 core file; not same as CCore above */
-#ifdef AFS_LINUX22_ENV
-#define CPageWrite      0x00200000      /* to detect vm deadlock - linux */
-#else
 #define CWritingUFS	0x00200000	/* to detect vm deadlock - used by sgi */
-#endif
 #define CCreating	0x00400000	/* avoid needless store after open truncate */
 #define CPageHog	0x00800000	/* AIX - dumping large cores is a page hog. */
 #define CDCLock		0x02000000	/* Vnode lock held over call to GetDownD */
@@ -710,6 +700,8 @@ struct vcache {
     afs_uint32 vstates;		/* vstate bits */
 #endif				/* defined(AFS_SUN5_ENV) */
     struct dcache *dchint;
+    struct dcache *dcreaddir;	/* dcache for in-progress readdir */
+    unsigned int readdir_pid;   /* pid of the thread in readdir */
 #ifdef AFS_LINUX22_ENV
     u_short mapcnt;		/* Number of mappings of this file. */
 #endif
@@ -847,6 +839,9 @@ struct cm_initparams {
 
 #define	NULLIDX	    (-1)	/* null index definition */
 /* struct dcache states bits */
+#define DRO         1
+#define DBackup     2
+#define DRW         4
 #define	DWriting    8		/* file being written (used for cache validation) */
 
 /* dcache data flags */
@@ -923,9 +918,6 @@ struct afs_fheader {
 #endif
 #endif
 
-
-#ifdef KERNEL
-/* it does not compile outside kernel */
 struct buffer {
   afs_int32 fid;              /* is adc->index, the cache file number */
   afs_inode_t inode;          /* is adc->f.inode, the inode number of the cac\
@@ -953,7 +945,6 @@ struct fcache {
     afs_int32 chunkBytes;	/* Num bytes in this chunk */
     char states;		/* Has this chunk been modified? */
 };
-#endif
 
 /* magic numbers to specify the cache type */
 
@@ -975,7 +966,7 @@ struct dcache {
     char dflags;		/* Data flags */
     char mflags;		/* Meta flags */
     struct fcache f;		/* disk image */
-
+    afs_int32 bucket;           /* which bucket these dcache entries are in */
     /*
      * Locking rules:
      *
@@ -1059,6 +1050,8 @@ struct memCacheEntry {
 /*#define afs_DirtyPages(avc)	(((avc)->states & CDirty) || osi_VMDirty_p((avc)))*/
 #define	afs_DirtyPages(avc)	((avc)->states & CDirty)
 
+#define afs_InReadDir(avc) (((avc)->states & CReadDir) && (avc)->readdir_pid == MyPidxx)
+
 /* The PFlush algorithm makes use of the fact that Fid.Unique is not used in
   below hash algorithms.  Change it if need be so that flushing algorithm
   doesn't move things from one hash chain to another
@@ -1231,9 +1224,4 @@ struct afs_fakestat_state {
 
 extern int afs_fakestat_enable;
 
-/* First 32 bits of capabilities */
-#define CAPABILITY_ERRORTRANS (1<<0)
-
-#define CAPABILITY_BITS 1
-
 #endif /* _AFS_H_ */
