@@ -18,7 +18,6 @@
 #define AFS_PPC64_LINUX24_ENV  1
 #define AFS_PPC64_LINUX26_ENV        1
 #define AFS_LINUX_64BIT_KERNEL 1
-#define AFS_NONFSTRANS 1
 
 #define AFS_MOUNT_AFS "afs"    /* The name of the filesystem type. */
 #define AFS_SYSCALL 137
@@ -58,7 +57,7 @@
 
 #define AFS_USERSPACE_IP_ADDR 1
 #define RXK_LISTENER_ENV 1
-#define AFS_GCPAGS       2 /* Set to Userdisabled, allow sysctl to override */
+#define AFS_GCPAGS       1
 
 /* Machine / Operating system information */
 #define SYS_NAME       "ppc64_linux26"
