@@ -19,7 +19,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vfsck/pass2.c,v 1.5 2003/07/15 23:17:27 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vfsck/pass2.c,v 1.7 2005/10/03 02:44:56 shadow Exp $");
 
 #define VICE
 #include <sys/time.h>
@@ -60,7 +60,11 @@ RCSID
 #include <sys/sysmacros.h>
 #include <sys/ino.h>
 #define	DIRSIZ_MACRO
+#ifdef HAVE_USR_OLD_USR_INCLUDE_NDIR_H
+#include </usr/old/usr/include/ndir.h>
+#else
 #include <ndir.h>
+#endif
 #else
 #include <sys/dir.h>
 #endif
@@ -198,12 +202,12 @@ pass2check(idesc)
     if (dirp->d_ino != 0 && strcmp(dirp->d_name, "..") != 0) {
 	pfatal("CANNOT FIX, FIRST ENTRY IN DIRECTORY CONTAINS %s\n",
 	       dirp->d_name);
-#if	defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV)
+#if	defined(AFS_SUN_ENV) 
 	iscorrupt = 1;
 #endif
     } else if (dirp->d_reclen < entrysize) {
 	pfatal("CANNOT FIX, INSUFFICIENT SPACE TO ADD '.'\n");
-#if	defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV)
+#if	defined(AFS_SUN_ENV) 
 	iscorrupt = 1;
 #endif
     } else if (dirp->d_reclen < 2 * entrysize) {
@@ -255,12 +259,12 @@ pass2check(idesc)
     if (dirp->d_ino != 0 && strcmp(dirp->d_name, ".") != 0) {
 	pfatal("CANNOT FIX, SECOND ENTRY IN DIRECTORY CONTAINS %s\n",
 	       dirp->d_name);
-#if	defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV)
+#if	defined(AFS_SUN_ENV) 
 	iscorrupt = 1;
 #endif
     } else if (dirp->d_reclen < entrysize) {
 	pfatal("CANNOT FIX, INSUFFICIENT SPACE TO ADD '..'\n");
-#if	defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV)
+#if	defined(AFS_SUN_ENV) 
 	iscorrupt = 1;
 #endif
     } else {
