@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/dumpstuff.c,v 1.29 2006/04/26 15:43:20 shadow Exp $");
+    ("$Header: /cvs/openafs/src/volser/dumpstuff.c,v 1.29.2.3 2006/10/13 13:09:07 shadow Exp $");
 
 #include <sys/types.h>
 #include <ctype.h>
@@ -524,9 +524,10 @@ DumpByteString(register struct iod *iodp, char tag, register byte * bs,
 static int
 DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 {
-    int code = 0, lcode = 0, error = 0;
+    int code = 0, error = 0;
     afs_int32 pad = 0, offset;
     afs_sfsize_t n, nbytes, howMany, howBig;
+    afs_foff_t lcode;
     byte *p;
 #ifndef AFS_NT40_ENV
     struct afs_stat status;
@@ -534,7 +535,12 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
     afs_sfsize_t size;
 #ifdef	AFS_AIX_ENV
 #include <sys/statfs.h>
+#if defined(AFS_AIX52_ENV) && defined(AFS_LARGEFILE_ENV)
+    struct statfs64 tstatfs;
+#else /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
     struct statfs tstatfs;
+#endif /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
+    int statfs_code;
 #endif
 
 #ifdef AFS_NT40_ENV
@@ -549,8 +555,17 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
     /* Unfortunately in AIX valuable fields such as st_blksize are 
      * gone from the stat structure.
      */
-    fstatfs(handleP->fd_fd, &tstatfs);
-    howMany = tstatfs.f_bsize;
+#if defined(AFS_AIX52_ENV) && defined(AFS_LARGEFILE_ENV)
+    statfs_code = fstatfs64(handleP->fd_fd, &tstatfs);
+#else /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
+    statfs_code = fstatfs(handleP->fd_fd, &tstatfs);
+#endif /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
+    if (statfs_code != 0) {
+        Log("DumpFile: fstatfs returned error code %d on descriptor %d\n", errno, handleP->fd_fd);
+	return VOLSERDUMPERROR;
+    }
+    howMany = (afs_sfsize_t) tstatfs.f_bsize;
+    Log("DumpFile: fstatfs returned block size of %lld; howMany=%lld", tstatfs.f_bsize, howMany);
 #else
     howMany = status.st_blksize;
 #endif /* AFS_AIX_ENV */
@@ -575,9 +590,9 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 	return VOLSERDUMPERROR;
     }
 
-    p = (unsigned char *)malloc(howMany);
+    p = (unsigned char *)malloc((size_t)howMany);
     if (!p) {
-	Log("1 Volser: DumpFile: no memory");
+	Log("1 Volser: DumpFile: not enough memory to allocate %u bytes\n", howMany);
 	return VOLSERDUMPERROR;
     }
 
@@ -586,7 +601,7 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 	    howMany = nbytes;
 
 	/* Read the data - unless we know we can't */
-	n = (lcode ? 0 : FDH_READ(handleP, p, howMany));
+	n = (lcode ? 0 : FDH_READ(handleP, p, (size_t)howMany));
 
 	/* If read any good data and we null padded previously, log the
 	 * amount that we had null padded.
@@ -620,7 +635,7 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 	    /* Now seek over the data we could not get. An error here means we
 	     * can't do the next read.
 	     */
-	    lcode = FDH_SEEK(handleP, ((size - nbytes) + howMany), SEEK_SET);
+	    lcode = FDH_SEEK(handleP, (size_t)((size - nbytes) + howMany), SEEK_SET);
 	    if (lcode != ((size - nbytes) + howMany)) {
 		if (lcode < 0) {
 		    Log("1 Volser: DumpFile: Error %d seeking in inode %s for vnode %d\n", errno, PrintInode(NULL, handleP->fd_ih->ih_ino), vnode);
@@ -634,7 +649,7 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 	}
 
 	/* Now write the data out */
-	if (iod_Write(iodp, (char *)p, howMany) != howMany)
+	if (iod_Write(iodp, (char *)p, (size_t)howMany) != howMany)
 	    error = VOLSERDUMPERROR;
 #ifndef AFS_PTHREAD_ENV
 	IOMGR_Poll();
@@ -1307,6 +1322,7 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 		 Error * status)
 {
     afs_int32 code;
+    afs_sfsize_t lcode;
     afs_fsize_t filesize;
     afs_fsize_t written = 0;
     register afs_uint32 size = 8192;
@@ -1352,11 +1368,11 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 	    *status = 3;
 	    break;
 	}
-	code = FDH_WRITE(handleP, p, size);
-	if (code > 0)
-	    written += code;
-	if (code != size) {
-	    Log("1 Volser: WriteFile: Error creating file in volume; restore aborted\n");
+	lcode = FDH_WRITE(handleP, p, size);
+	if (lcode > 0)
+	    written += lcode;
+	if (lcode != size) {
+	    Log("1 Volser: WriteFile: Error writing (%d,%u) bytes to vnode %d; restore aborted\n", (int)(lcode>>32), (int)(lcode & 0xffffffff), vn);
 	    *status = 4;
 	    break;
 	}
